/*
 * Copyright (C) 2012-present the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.pf4j;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.io.TempDir;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * @author Mario Franco
 * @author Decebal Suiu
 */
public class ManifestPluginDescriptorFinderTest {

    private VersionManager versionManager;

    @TempDir
    Path pluginsPath;

    @BeforeEach
    public void setUp() throws IOException {
        Charset charset = Charset.forName("UTF-8");

        Path pluginPath = Files.createDirectories(pluginsPath.resolve(Paths.get("test-plugin-1", "classes", "META-INF")));
        Files.write(pluginPath.resolve("extensions.idx"), "org.pf4j.demo.hello.HelloPlugin$HelloGreeting".getBytes());
        Files.write(pluginPath.resolve("MANIFEST.MF"), getPlugin1Manifest(), charset);

        pluginPath = Files.createDirectories(pluginsPath.resolve(Paths.get("test-plugin-2", "classes", "META-INF")));
        Files.write(pluginPath.resolve("extensions.idx"), "org.pf4j.demo.hello.HelloPlugin$HelloGreeting".getBytes());
        Files.write(pluginPath.resolve("MANIFEST.MF"), getPlugin2Manifest(), charset);

        // empty plugin
        Files.createDirectories(pluginsPath.resolve("test-plugin-3"));

        // no plugin class
        pluginPath = Files.createDirectories(pluginsPath.resolve(Paths.get("test-plugin-4", "classes", "META-INF")));
        Files.write(pluginPath.resolve("extensions.idx"), "org.pf4j.demo.hello.HelloPlugin$HelloGreeting".getBytes());
        Files.write(pluginPath.resolve("MANIFEST.MF"), getPlugin4Manifest(), charset);

        // no plugin version
        pluginPath = Files.createDirectories(pluginsPath.resolve(Paths.get("test-plugin-5", "classes", "META-INF")));
        Files.write(pluginPath.resolve("extensions.idx"), "org.pf4j.demo.hello.HelloPlugin$HelloGreeting".getBytes());
        Files.write(pluginPath.resolve("MANIFEST.MF"), getPlugin5Manifest(), charset);

        // no plugin id
        pluginPath = Files.createDirectories(pluginsPath.resolve(Paths.get("test-plugin-6", "classes", "META-INF")));
        Files.write(pluginPath.resolve("extensions.idx"), "org.pf4j.demo.hello.HelloPlugin$HelloGreeting".getBytes());
        Files.write(pluginPath.resolve("MANIFEST.MF"), getPlugin6Manifest(), charset);

        versionManager = new DefaultVersionManager();
    }

    /**
     * Test of {@link ManifestPluginDescriptorFinder#find(Path)} method.
     */
    @Test
    public void testFind() throws Exception {
        PluginDescriptorFinder descriptorFinder = new ManifestPluginDescriptorFinder();

        PluginDescriptor plugin1 = descriptorFinder.find(pluginsPath.resolve("test-plugin-1"));
        PluginDescriptor plugin2 = descriptorFinder.find(pluginsPath.resolve("test-plugin-2"));

        assertEquals("test-plugin-1", plugin1.getPluginId());
        assertEquals("Test Plugin 1", plugin1.getPluginDescription());
        assertEquals("org.pf4j.plugin.TestPlugin", plugin1.getPluginClass());
        assertEquals("0.0.1", plugin1.getVersion());
        assertEquals("Decebal Suiu", plugin1.getProvider());
        assertEquals(2, plugin1.getDependencies().size());
        assertEquals("test-plugin-2", plugin1.getDependencies().get(0).getPluginId());
        assertEquals("test-plugin-3", plugin1.getDependencies().get(1).getPluginId());
        assertEquals("~1.0", plugin1.getDependencies().get(1).getPluginVersionSupport());
        assertEquals("Apache-2.0", plugin1.getLicense());
        assertTrue(versionManager.checkVersionConstraint("1.0.0", plugin1.getRequires()));

        assertEquals("test-plugin-2", plugin2.getPluginId());
        assertEquals("", plugin2.getPluginDescription());
        assertEquals("org.pf4j.plugin.TestPlugin", plugin2.getPluginClass());
        assertEquals("0.0.1", plugin2.getVersion());
        assertEquals("Decebal Suiu", plugin2.getProvider());
        assertEquals(0, plugin2.getDependencies().size());
        assertTrue(versionManager.checkVersionConstraint("1.0.0", plugin2.getRequires()));
    }

    /**
     * Test of {@link ManifestPluginDescriptorFinder#find(Path)} method.
     */
    @Test
    public void testFindNotFound() {
        PluginDescriptorFinder descriptorFinder = new ManifestPluginDescriptorFinder();
        assertThrows(PluginException.class, () -> descriptorFinder.find(pluginsPath.resolve("test-plugin-3")));
    }

    private List<String> getPlugin1Manifest() {
        String[] lines = new String[] {
            "Manifest-Version: 1.0\n"
            + "Implementation-Title: Test Plugin #1\n"
            + "Implementation-Version: 0.10.0-SNAPSHOT\n"
            + "Archiver-Version: Plexus Archiver\n"
            + "Built-By: Mario Franco\n"
            + "Specification-Title: Test Plugin #1\n"
            + "Implementation-Vendor-Id: org.pf4j.demo\n"
            + "Plugin-Version: 0.0.1\n"
            + "Plugin-Id: test-plugin-1\n"
            + "Plugin-Description: Test Plugin 1\n"
            + "Plugin-Provider: Decebal Suiu\n"
            + "Plugin-Class: org.pf4j.plugin.TestPlugin\n"
            + "Plugin-Dependencies: test-plugin-2,test-plugin-3@~1.0\n"
            + "Plugin-Requires: *\n"
            + "Plugin-License: Apache-2.0\n"
            + "Created-By: Apache Maven 3.0.5\n"
            + "Build-Jdk: 1.8.0_45\n"
            + "Specification-Version: 0.10.0-SNAPSHOT\n"
            + "\n"
            + ""
        };

        return Arrays.asList(lines);
    }

    private List<String> getPlugin2Manifest() {
        String[] lines = new String[] {
            "Manifest-Version: 1.0\n"
            + "Plugin-Dependencies: \n"
            + "Implementation-Title: Test Plugin #2\n"
            + "Implementation-Version: 0.10.0-SNAPSHOT\n"
            + "Archiver-Version: Plexus Archiver\n"
            + "Built-By: Mario Franco\n"
            + "Specification-Title: Test Plugin #2\n"
            + "Implementation-Vendor-Id: org.pf4j.demo\n"
            + "Plugin-Version: 0.0.1\n"
            + "Plugin-Id: test-plugin-2\n"
            + "Plugin-Provider: Decebal Suiu\n"
            + "Plugin-Class: org.pf4j.plugin.TestPlugin\n"
            + "Created-By: Apache Maven 3.0.5\n"
            + "Build-Jdk: 1.8.0_45\n"
            + "Specification-Version: 0.10.0-SNAPSHOT\n"
            + "\n"
            + ""
        };

        return Arrays.asList(lines);
    }

    private List<String> getPlugin4Manifest() {
        String[] lines = new String[] {
            "Manifest-Version: 1.0\n"
            + "Implementation-Title: Test Plugin #4\n"
            + "Implementation-Version: 0.10.0-SNAPSHOT\n"
            + "Archiver-Version: Plexus Archiver\n"
            + "Built-By: Mario Franco\n"
            + "Specification-Title: Test Plugin #4\n"
            + "Implementation-Vendor-Id: org.pf4j.demo\n"
            + "Plugin-Version: 0.0.1\n"
            + "Plugin-Id: test-plugin-2\n"
            + "Plugin-Provider: Decebal Suiu\n"
            + "Created-By: Apache Maven 3.0.5\n"
            + "Build-Jdk: 1.8.0_45\n"
            + "Specification-Version: 0.10.0-SNAPSHOT\n"
            + "\n"
            + ""
        };

        return Arrays.asList(lines);
    }

    private List<String> getPlugin5Manifest() {
        String[] lines = new String[] {
            "Manifest-Version: 1.0\n"
            + "Implementation-Title: Test Plugin #5\n"
            + "Implementation-Version: 0.10.0-SNAPSHOT\n"
            + "Archiver-Version: Plexus Archiver\n"
            + "Built-By: Mario Franco\n"
            + "Specification-Title: Test Plugin #5\n"
            + "Implementation-Vendor-Id: org.pf4j.demo\n"
            + "Plugin-Id: test-plugin-2\n"
            + "Plugin-Provider: Decebal Suiu\n"
            + "Plugin-Class: org.pf4j.plugin.TestPlugin\n"
            + "Created-By: Apache Maven 3.0.5\n"
            + "Build-Jdk: 1.8.0_45\n"
            + "Specification-Version: 0.10.0-SNAPSHOT\n"
            + "\n"
            + ""
        };

        return Arrays.asList(lines);
    }

    private List<String> getPlugin6Manifest() {
        String[] lines = new String[] {
            "Manifest-Version: 1.0\n"
            + "Implementation-Title: Test Plugin #6\n"
            + "Implementation-Version: 0.10.0-SNAPSHOT\n"
            + "Archiver-Version: Plexus Archiver\n"
            + "Built-By: Mario Franco\n"
            + "Specification-Title: Test Plugin #6\n"
            + "Implementation-Vendor-Id: org.pf4j.demo\n"
            + "Plugin-Provider: Decebal Suiu\n"
            + "Plugin-Class: org.pf4j.plugin.TestPlugin\n"
            + "Created-By: Apache Maven 3.0.5\n"
            + "Build-Jdk: 1.8.0_45\n"
            + "Specification-Version: 0.10.0-SNAPSHOT\n"
            + "\n"
            + ""
        };

        return Arrays.asList(lines);
    }

}
